/*
 * annotations.js
 * Pedro Fonseca, 2010-2012
 */
var lineNumberElementsList = null;
var lineNumberElement = null;
var blameTextLength = 0;
var swipeStatus = -1;
var startX = 0;
var startY = 0;
var active = 0;


function discardAnnotation(e) {
    document.getElementById('textbox').style.display = 'none';
    lineNumberElement.style.border = 0;
    lineNumberElement = null;
    document.editAnnotation.reset();
    return true;
}

function onClickEditWrapper(e) {
    lineClickHandler(e.target, 1);
}

function onClickNewWrapper(e) {
    lineClickHandler(e.target, 0);
}

function attachAnnotationsEvents(element) {
    active = 1;
    
    element.addEventListener('click', onClickEditWrapper, false);
    element.addEventListener('touchstart', touchStart, false);
    element.addEventListener('touchmove', touchMove, false);
    element.addEventListener('touchend', touchEnd, false);
    element.addEventListener('touchcancel', touchCancel, false);
}

function detachAnnotationsEvents(element) {
    element.removeEventListener('click', onClickEditWrapper, false);
    element.removeEventListener('touchstart', touchStart, false);
    element.removeEventListener('touchmove', touchMove, false);
    element.removeEventListener('touchend', touchEnd, false);
    element.removeEventListener('touchcancel', touchCancel, false);
    
    active = 0;
}

/* 
 * Attach events to existing annotations. This is straightforward because the annotations are the 
 * only elements that use <p> tags. This doesn't attach events to the line number field, which is 
 * best handled by the parser, since every element other inside <pre> uses the span tag, so we 
 * would have to inspect the class of each element.
 */
function attachEvents() {
    var annotations = document.getElementsByTagName('pre')[0].getElementsByTagName('p');
    for (i in annotations) {
        attachAnnotationsEvents(annotations[i]);
    }
}

function detachEvents() {
    var annotations = document.getElementsByTagName('pre')[0].getElementsByTagName('p');
    for (i in annotations) {
        detachAnnotationsEvents(annotations[i]);
    }
}   

function saveAnnotation(e) {
    var node;
    var form = document.editAnnotation;
    
    var annotationStyle = 'd' + form.flag.value;
    
    // The form action string is different starting with iOS 5.
    var newRe = /(^|([\w]*(:\/\/)?[^/]*\/)(NEW))$/;
    var editRe = /(^|([\w]*(:\/\/)?[^/]*\/)(EDIT))$/;
    
    // Sometimes the UIWebView receives a null request. This can't be reproduced, so we make sure 
    // the form action is set here.
    if (form.action != 'EDIT' && form.action != 'NEW' && 
        !editRe.test(form.action) && !newRe.test(form.action)) {
        form.action = 'EDIT';
    }
   
    if (form.action === 'EDIT' || editRe.test(form.action)) {
        node = lineNumberElement.previousSibling;
        if (node.tagName != 'P' && node.tagName != 'p') {
            form.action = 'NEW';
        }
    } 
    
    if (form.action === 'NEW' || newRe.test(form.action)) {
        node = document.createElement('p');
        
        var parent = lineNumberElement.parentNode;
        node = parent.insertBefore(node, lineNumberElement);
    }
    
    // Break lines to fit on the screen in portrait mode (...cannot be done with css overflow).
    var linebreak;
    if (screen.width > 480) {
        linebreak = 84;
    } else {
        linebreak = 36;
    }
     
    var text = '';
    var lines = form.text.value.split('\n');
    for (i in lines) {
        var s = lines[i];
        
        var start = 0;
        var offset = 0;
        while (s.length - offset > linebreak)
        {
            offset += linebreak - 1;
            while (offset < s.length && s[offset] != ' ' && s[offset] != '\t') {
                ++offset;
            }
            if (offset < s.length) {
                text += s.substr(start, offset) + '\n';
                start = offset + 1;
            }
        }
    
        text += s.substr(start, s.length);
        if (text.length > 0 && text[text.length - 1] != '\n') {
            text += '\n';
        }
    }    
    
    
    node.innerHTML = text;
    node.className = 'x ' + annotationStyle;
    lineNumberElement.className = 'l ' + annotationStyle;
    
    lineNumberElement.addEventListener('click', onClickEditWrapper, false);
    attachAnnotationsEvents(node);
    
    document.getElementById('textbox').style.display = 'none';
    
    lineNumberElement.style.border = 0;
    lineNumberElement = null;
    
    form.submit();
    return true;
}


/* Swipe gesture support */

function touchStart(event) {
    if (event.targetTouches.length == 1) {
        swipeStatus = 0;
        startX = event.targetTouches[0].pageX;
        startY = event.targetTouches[0].pageY;
    } else {
        swipeStatus = -1;
    }
}

function touchMove(event) {
    if (swipeStatus < 0)
        return;
       
    event.preventDefault();
    
    offX = event.targetTouches[0].pageX - startX;
    offY = event.targetTouches[0].pageY - startY;
    
    if (Math.abs(offX) > Math.abs(offY)) {
        swipeStatus = 1;
    } else {
        swipeStatus = 0;
    }
}

function touchCancel(event) {
    swipeStatus = -1;
}

function touchEnd(event) {
    if (swipeStatus == 1) {
        var remove = confirm('Remove annotation?'); 
        if (remove)
        {
            var annotationElement = event.target;
            if (event.target.nodeValue != null) {
                // humm... this is a text node, what we want to remove is the parent element
                annotationElement = event.target.parentNode;
            }
        
            lineNumberElement = annotationElement.nextSibling;
            var lineNumber = lineNumberElement.innerHTML;
            lineNumber = lineNumber.replace(/[^\d]/g, '');
            
            // If the form is used for REMOVE requests, form.submit() stops working at the second 
            // consecutive call.
            document.location.href = '/REMOVE?flag=&text=&line=' + lineNumber;
            
            lineNumberElement.className = 'l';
            lineNumberElement.onclick = onClickNewWrapper;
            lineNumberElement = null;
            
            annotationElement.parentNode.removeChild(annotationElement);
        }
    }
    swipeStatus = -1;
}


/* 0 == New Annotation; 1 == Edit Annotation */
function lineClickHandler(obj, op) {
    if (active === 0)
        return;
        
    var form = document.editAnnotation;
    var textbox = document.getElementById('textbox');
    
    // If the textbox is already displayed we need to switch the number.
    if (textbox.style.display == 'block') {
        // This is the previous line number element.
        if (lineNumberElement != null)
            lineNumberElement.style.border = '0';
    }
    
    document.getElementById('save').onclick = saveAnnotation;
    document.getElementById('cancel').onclick = discardAnnotation;    
    
    // Parse the line number contained in the clicked element inner text, if the element is the 
    // lineNumberElement. If the element is the annotation itself, the line number is contained in 
    // the next sibling.
    if (obj.tagName == 'P' || obj.tagName == 'p') {
        obj = obj.nextSibling;
    }
    lineNumberElement = obj;
    
    var lineNumber = obj.innerHTML;
    lineNumber = lineNumber.replace(/[^\d]/g, '');
    form.line.value = lineNumber;
   
    lineNumberElement.style.border = '1 px solid blue';
    
    if (op == 1) {
        form.text.value = obj.previousSibling.innerHTML;
        form.action = 'EDIT';
        
        // Find the annotation type from the class name.
        var type = obj.previousSibling.className.match(/[1-3]$/);
        form.flag.value = type;
    } else {
        form.action = 'NEW';
        form.text.value = '';
    }
    
    // Detect the iOS version to work around UIWebView implementation changes.
    var matched = navigator.userAgent.match(/OS (\d)_(.)/i);
    if (matched) {
        var majorVersion = parseInt(matched[1], 10);
        var minorVersion = parseInt(matched[2], 10);
    }
    
    if(majorVersion > 4) {
        textbox.style.position = 'absolute';
        textbox.style.top = window.pageYOffset + 8 + 'px';
    } else {
        textbox.style.top = window.pageYOffset + 8 + 'px';
    }
    textbox.style.display = 'block';
}

/* Push blame information to a loaded page. */
function addBlameInfo(lineNumber, blameText) {
    if (lineNumber === 0) {  
        lineNumberElementsList = document.getElementsByTagName('pre')[0].getElementsByClassName('l');
        blameTextLength = blameText.length;
    }
    
    var infoNode = document.createElement('span');
    infoNode.appendChild(document.createTextNode(blameText));
    infoNode.className = 'a';
    
    if (lineNumberElementsList[lineNumber].nextSibling.className == 'a') {
        lineNumberElementsList[lineNumber].nextSibling.innerHTML = blameText;
    } else {
        var parent = lineNumberElementsList[lineNumber].parentNode;
        parent.insertBefore(infoNode, lineNumberElementsList[lineNumber].nextSibling);
    }
    
    if (lineNumber === lineNumberElementsList.length - 1) {
        lineNumberElementsList = null;
    }
}

function padTrailingLine(lineNumber) {
    if (lineNumberElementsList != null && lineNumber === lineNumberElementsList.length - 1) {
        var padding = '';
        for (; blameTextLength > 0; --blameTextLength)
            padding += ' ';
        
        addBlameInfo(lineNumber, padding);
    }
}


/* Aliasing. */
window.lch = lineClickHandler;

