/* cvfm.js 2009-2011 Pedro Fonseca */
var _ids = ['upload', 'directories', 'help'];
var _act = 'upload';
var _timer;

function setActive(id, tab) {
    document.getElementById(id).style.display = 'block';
    tab.className = 'activeSection'; 
    _act = id;	
}

function switchTo(id, tab) {			
    for (var i = 0; i < _ids.length; i++) {
        document.getElementById(_ids[i]).style.display = 'none';
        document.getElementById('t' + i).className = '';
    }
    setActive(id, tab);
}

function setQuery(evt) {
    var elem;
    var index;
    if (!evt) {
        elem = window.event.srcElement;
        for (var i in _ids)
            if (_ids[i] === _act) {
                index = i;
                break;
            }
    } else {
        elem = this;
        index = _ids.indexOf(_act);
    }
        
    if (elem.href.search(/\?[^t]{1}.*/) != -1)
        elem.href = elem.href.replace(/&t.*|$/, '&t' + index);
    else
        elem.href = elem.href.replace(/\?.*|$/, '?t' + index);
        
    return true;
}

function setup() {
    var elems = document.getElementsByTagName('input');
    for (var i = 0; i < elems.length; ++i)
        if (elems[i].name == 'CurrentPath')
            elems[i].value = window.location.pathname;
}

function setupTabs() {
    var links = document.getElementById('list').getElementsByTagName('a');
    for (var i in links)
        links[i].onclick = setQuery;
    
    var q = window.location.search.substring(1);
    if (q && q.search(/t\d$/) != -1)
        switchTo(_ids[q.charAt(q.length - 1)], document.getElementById(q));
}

function updateNote(parent, result) {
    var node = document.createElement('p');
    if (result['error'] === 'true')
        node.className = 'error'; 
    else 
        node.className = 'note';
    
    node.appendChild(document.createTextNode(result['msg']));
    while (parent.hasChildNodes()) {
        parent.removeChild(parent.firstChild);
    }
    parent.appendChild(node);
}

function addToFileList(result) {
    var name = result['name'];
    var path = result['path'];
    
    var filelist = document.getElementsByTagName('table')[0];
    var tr = document.createElement('tr');
    tr.className = 'recent';
    for (var k = 1; k <= 4; ++k) {
        var td = document.createElement('td');
        
        switch (k) {
            case 1:
                td.className = 'type';
                td.appendChild(document.createTextNode('?'));
                break;
            case 2: {
                td.className = 'name';
                var a = document.createElement('a');
                while (name.charAt(0) == '/') name = name.substring(1);
                if (path == '/') path = '';
                a.href = path + '/' + name;
                a.onclick = setQuery;
                a.appendChild(document.createTextNode(name)); 
                td.appendChild(a);
            }
                break;
            case 4:
                td.appendChild(document.createTextNode('Just created'));
                break;
            default:
                td.appendChild(document.createTextNode('n/a'));
                break;
        }
        
        tr.appendChild(td);
    }
    filelist.appendChild(tr);
}

function createDir() {
    var fields = document.getElementById('dirform').elements;
    var currentpath = encodeURIComponent(fields['CurrentPath'].value);
    var dirname = fields['DirName'].value;
    
    dirname = dirname.replace(/^\s+|\s+$/, '');
    dirname = dirname.replace(/^[/]*([.]{1,2}[/]|$)*/g, '');
    while (dirname.charAt(0) == '/') dirname = dirname.substring(1);
    if (dirname.length == 0 || dirname === '.' || dirname === '..') {
        document.getElementById('notifications').innerHTML = '<p class="error">Invalid name!</p>';
        fields['DirName'].value = '';
        return;
    }
    dirname = encodeURIComponent(dirname);
    
    var xhr = new XMLHttpRequest();
    xhr.open('GET', '/createdir.cv?DirName=' + dirname + '&CurrentPath=' + currentpath, true);
    xhr.onreadystatechange = function(evt) {
        if (xhr.readyState == 4) {
            var elem = document.getElementById('notifications');
            if (xhr.status == 200) {
                var results = window.JSON ? JSON.parse(xhr.responseText) : eval('(' + xhr.responseText + ')');
                updateNote(elem, results);
                if (results['error'] === 'false') {
                    addToFileList(results);
                }
                fields['DirName'].value = '';
            } else {
                elem.innerHTML = '<p class="error">Error requesting operation (' + xhr.status + ')</p>';
            }
            document.getElementById('dirform').elements['DoCreateDir'].disabled = false;
        }
    }
    
    xhr.send(null);
    fields['DoCreateDir'].disabled = true;
}

function enableOldUpload() {
    var oldContents = document.getElementById('old_upload').innerHTML;
    document.getElementById('upload').innerHTML = oldContents;
}

function startUpload(inputField) {
    var f = document.getElementById('uploadFrame');
    if (f.addEventListener)
        f.addEventListener('load', didUpload, false);
    else 
        f.attachEvent('onload', didUpload);
        
    inputField.form.submit();
    
    // The server resets the connections after responding with a 405 for POST requests exceed 1GB.
    //   Firefox just jumps to the connection reset message and the load event is not triggered;
    //   Google Chrome, IE and Opera trigger the load event;
    //   and Safari just keeps loading...
    _timer = setInterval(checkReset, 15000);
    
    document.getElementById('filesform').elements['File1'].disabled = true;
    document.getElementById('uploadStatus').innerHTML = 'Uploading ' + inputField.value 
        + '... <a href="#" onclick="abortUpload()" target="uploadFrame">Cancel</a>';
}

function abortUpload() {
    clearInterval(_timer);
    var f = document.getElementById('uploadFrame');
    if (f.removeEventListener)
        f.removeEventListener('load', didUpload, false);
    else
        f.detachEvent('onload', didUpload);
    
    if (window.stop)
        f.contentWindow.stop();
    else
        f.contentWindow.document.execCommand('Stop');
    
    document.getElementById('notifications').innerHTML = '<p>Upload canceled</p>';
    resetFileUpload();
}

function didUpload() {
    clearInterval(_timer);
    var f = window.parent.document.getElementById('uploadFrame');
    if (f.removeEventListener)
        f.removeEventListener('load', didUpload, false);
    else
        f.detachEvent('onload', didUpload);
    
    try {
        var d = f.contentWindow.document.documentElement;
        var t = d.textContent || d.innerText;
        var results = window.JSON ? JSON.parse(t) : eval('(' + t + ')');
        updateNote(document.getElementById('notifications'), results);
        if (results['error'] === 'false') {
            addToFileList(results);
        }
    } catch (e) {
        document.getElementById('notifications').innerHTML = '<p class="error">Unspecified error. Chances are that the connection was reset or the file size exceeded 1GB</p>';
    }
    resetFileUpload();
}

function resetFileUpload() {
    document.getElementById('uploadStatus').innerHTML = '';
    
    var form = document.getElementById('filesform');
    form.elements['File1'].disabled = false;
    form.reset();
    
    document.body.removeChild(document.getElementById('uploadFrame'));
    var frame = document.createElement('iframe');
    frame.id = 'uploadFrame';
    frame.name = 'uploadFrame';
    document.body.appendChild(frame);
}


function checkReset() {
    var f = document.getElementById('uploadFrame');
    var o = f.contentWindow;
    try {
        // If the connection was reset accessing o.document throws an exception in Firefox.
        o.document;
    } catch (e) {
        didUpload();
    }
}
